#include <SPI.h>
#include <MFRC522.h>
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>

// Pins for RFID
#define RST_PIN  D1 // Reset pin
#define SS_PIN   D2 // Slave Select pin

// LED Pins
#define GREEN_LED D5
#define RED_LED   D6

MFRC522 rfid(SS_PIN, RST_PIN);

// Wi-Fi credentials
const char* ssid = "Your_WiFi_SSID";        // Replace with your Wi-Fi SSID
const char* password = "Your_WiFi_Password"; // Replace with your Wi-Fi password

// Google Apps Script URL
const char* googleScriptURL = "https://script.google.com/macros/s/pasteyourscripturl"; //paste your script url

void setup() {
  Serial.begin(9600); // Initialize Serial Monitor
  SPI.begin();        // Initialize SPI bus
  rfid.PCD_Init();    // Initialize RFID reader

  // Initialize LED pins
  pinMode(GREEN_LED, OUTPUT);
  pinMode(RED_LED, OUTPUT);
  digitalWrite(GREEN_LED, LOW);
  digitalWrite(RED_LED, LOW);

  Serial.println("Connecting to Wi-Fi...");
  WiFi.begin(ssid, password);

  // Wait for Wi-Fi connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.print(".");
  }
  Serial.println("\nWi-Fi Connected!");
}

void loop() {
  // Check if an RFID card is present
  if (!rfid.PICC_IsNewCardPresent() || !rfid.PICC_ReadCardSerial()) {
    return;
  }

  // Read the RFID card's UID
  String cardID = "";
  for (byte i = 0; i < rfid.uid.size; i++) {
    cardID += String(rfid.uid.uidByte[i], HEX);
  }
  cardID.toUpperCase();

  Serial.println("Card ID: " + cardID);

  // Send the card ID to Google Apps Script
  if (WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    String url = String(googleScriptURL) + "?cardID=" + cardID;
    http.begin(url);

    int httpResponseCode = http.GET();
    if (httpResponseCode > 0) {
      String response = http.getString();
      Serial.println("Response: " + response);

      // Turn on the green LED for success
      digitalWrite(GREEN_LED, HIGH);
      delay(1000); // Keep the LED on for 1 second
      digitalWrite(GREEN_LED, LOW);
    } else {
      Serial.println("Error: Unable to send data");

      // Turn on the red LED for failure
      digitalWrite(RED_LED, HIGH);
      delay(1000); // Keep the LED on for 1 second
      digitalWrite(RED_LED, LOW);
    }
    http.end();
  } else {
    Serial.println("Wi-Fi Disconnected!");

    // Blink the red LED to indicate Wi-Fi issue
    for (int i = 0; i < 3; i++) {
      digitalWrite(RED_LED, HIGH);
      delay(200);
      digitalWrite(RED_LED, LOW);
      delay(200);
    }
  }

  // Halt RFID reading until the next card
  rfid.PICC_HaltA();
}
